/*
        Mojodo Project Code (c)
        Project: Socket Raw Library
        Coders: ValK
        thxto: Harada
*/

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/ip_icmp.h>
#include <netinet/udp.h>

#include <linux/if_arp.h>
#include <linux/if_ether.h>

/* v6 Header */
#include <netinet/ip6.h>
#include <netinet/icmp6.h>

#include <linux/sockios.h>

#define __GNU_SOURCE
#define LIBVERSION "2.1"
#define MTU 1500
#define NODATA ""

/*#define DEBUG*/

const        char    libinfo[]="Mojodo Project Code ,no (c) 2002";
const        char    liblink[]="Avatar Corp. ,no (c) 2002";
const        char    license[]="GNU/GPL";
const        int     TCP_MAXDATALEN=(MTU-sizeof(struct tcphdr)) - sizeof(struct iphdr);
const        int     ICMP_MAXDATALEN=(MTU-sizeof(struct icmphdr)) - sizeof(struct iphdr);
const        int     UDP_MAXDATALEN=(MTU-sizeof(struct udphdr)) - sizeof(struct iphdr);
const        int     TCP6_MAXDATALEN=(MTU-sizeof(struct tcphdr)) - sizeof(struct ip6_hdr);
const        int     ICMP6_MAXDATALEN=(MTU-sizeof(struct icmp6_hdr)) - sizeof(struct ip6_hdr);
const        int     UDP6_MAXDATALEN=(MTU-sizeof(struct udphdr)) - sizeof(struct ip6_hdr);

struct ls_v6opt {
	int  flow_label,
	     hop_limit,
	     vfc;
	char device[10];
	};

struct ls_settings
        {
        int     seq,
                ack_seq,
                window,
                urg_ptr;
        };

struct ls_tcpflags
{
        int     doff,
                res1,
                res2,
                urg,
                ack,
                psh,
                rst,
                syn,
                fin;
};

struct ls_icmp6 {
	u_int8_t  type,
	          code;
	u_int16_t id,
                  seq;
	int       pptr,
	          mtu;
};
struct ls_ip {
    u_int8_t tos;
    u_int16_t id;
    u_int16_t frag_off;
    u_int8_t ttl;
};

struct pseudoIP
{
        unsigned long int source, dest;
        char zero_byte, protocol;
        unsigned short len;
};
struct pseudoIP6 {
	struct in6_addr src;
	struct in6_addr dst;
	uint32_t ulpl;
	uint32_t zero_nxt;
};

/**[-ARP by Lupsyn-]***********************************/
struct ls_arp
{
        u_char Dst_e_hw[ETH_ALEN];
        u_char Src_e_hw[ETH_ALEN];
        u_short frame_type;
        u_short hw_type;
        u_short prot_type;
        u_char hw_addr_size;
        u_char prot_addr_size;
        u_short op;
        u_char Src_a_hw[ETH_ALEN];
        u_char Src_a_ip[4];
        u_char Dst_a_hw[ETH_ALEN];
        u_char Dst_a_ip[4];
};
/****************************************************/

struct sockaddr_in sockfd;


int sock;

/* Vari Prototipi */
in_addr_t inet_addr(const char *cp);
int inet_pton(int af, const char *src, void *dst);
int close(int fd);

char *libversion();
int ls_socket(char *dstHost);
int ls_sendtcp(struct ls_ip *ip4,char *srcHost,char *dstHost,int sport, int dport,unsigned char *data, int binlen , struct ls_tcpflags flags, struct ls_settings settings);
int ls_sendicmp(struct ls_ip *ip4,char *srcHost,char *dstHost, u_int8_t type, u_int8_t code,unsigned char *data, int binlen);
int ls_sendudp(struct ls_ip *ip4,char *srcHost,char *dstHost,int sport, int dport,unsigned char *data, int binlen);

/* v6 Funcion */
int ls_socket6(char *dstHost, char *device);
int ls_sendudp6(char *srcHost,char *dstHost,int sport, int dport,unsigned char *data, int binlen,struct ls_v6opt option);
int ls_sendtcp6(char *srcHost,char *dstHost,int sport, int dport,unsigned char *data, int binlen, struct ls_tcpflags flags, struct ls_settings settings, struct ls_v6opt option);
int ls_sendicmp6(char *srcHost,char *dstHost,struct ls_icmp6 icmpinfo,unsigned char *data, int binlen, struct ls_v6opt option);
void setiphdr6(struct ip6_hdr *ip6, int iplen, int protocol,char *srcHost, char *dstHost,int frag, int datalen,struct ls_v6opt option);

/* ARP by Lupsyn */
void die(char* str);
void ls_sendarp(struct ls_arp pkt, char *dev);
int getdevindex(int sock, char *name);

void setiphdr(struct iphdr *ip,struct ls_ip *ip4, int iplen, int protocol,char *srcHost, char *dstHost,int frag, int datalen);
unsigned short in_cksum(unsigned short *addr,register unsigned int len,int csum);

char *
libversion()
{
        return(LIBVERSION);
}

int
ls_socket(char *dstHost)
{
        int on=1;

        memset(&sockfd, 0, sizeof(struct sockaddr_in));

        if ((sock = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0) {
                perror("socket");
                exit(-1);
        }

        sockfd.sin_family = AF_INET;
        sockfd.sin_port = htons(0);

        if (inet_addr(dstHost) == INADDR_NONE)
                exit(-1);

        sockfd.sin_addr.s_addr = inet_addr(dstHost);

        if (setsockopt(sock, IPPROTO_IP, IP_HDRINCL, &on, sizeof(on)) < 0) {
                perror("setsockopt() ");
                exit(-1);
        }

        return(sock);
}

int
ls_sendtcp(struct ls_ip *ip4,char *srcHost,char *dstHost,int sport, int dport,unsigned char *data, int binlen, struct ls_tcpflags flags, struct ls_settings settings)
{

        struct iphdr *ip;
        struct tcphdr *tcp;
        unsigned char *msg;
        unsigned char packet[1500];

        struct pseudoIP psdIP;
        unsigned short *psdHdr;
        unsigned int csum=0;
        int len=strlen(data), nodata=0;

        if(!strlen(data))
                nodata++;
        if(binlen)
                len=binlen;

        if (len > TCP_MAXDATALEN ) {
                printf("Data Field too big.\n");
                exit(-1);
        }


        sock=ls_socket(dstHost);

        memset(packet, 0, 1500);

        ip = (struct iphdr *)packet;

        tcp = (struct tcphdr *)(packet + sizeof(struct iphdr));

        msg = (unsigned char *)(packet + sizeof(struct iphdr) + sizeof(struct tcphdr));

        setiphdr(ip,ip4,sizeof(struct tcphdr) + sizeof(struct iphdr), IPPROTO_TCP,srcHost, dstHost,0,0);

        memset(&psdIP, 0, sizeof(struct pseudoIP));

        psdIP.source=ip->saddr;
        psdIP.dest=ip->daddr;
        psdIP.zero_byte=0;
        psdIP.protocol=IPPROTO_TCP;
        psdIP.len=htons(sizeof(struct tcphdr)+ len);

        psdHdr=(unsigned short * )&psdIP;
        csum = (psdHdr[0] + psdHdr[1] + psdHdr[2] + psdHdr[3] + psdHdr[4] + psdHdr[5]);

        #ifdef DEBUG
        fprintf(stdout,"\nDebug Mode [ON]\n");
        fprintf(stdout,"doff %d\n",flags.doff);
        fprintf(stdout,"res1 %d\n",flags.res1);
        fprintf(stdout,"res2 %d\n",flags.res2);
        fprintf(stdout,"usg %d\n",flags.urg);
        fprintf(stdout,"ack %d\n",flags.ack);
        fprintf(stdout,"psh %d\n",flags.psh);
        fprintf(stdout,"rst %d\n",flags.rst);
        fprintf(stdout,"fin %d\n",flags.fin);
        fprintf(stdout,"syn %d\n",flags.syn);
        fprintf(stdout,"ece %d\n",flags.ece);
        fprintf(stdout,"cwr %d\n",flags.cwr);
        fprintf(stdout,"ack_seq %d\n",settings.ack_seq);
        fprintf(stdout,"seq %d\n",settings.seq);
        fprintf(stdout,"win %d\n",settings.window);
        #endif
        tcp->doff=(u_int16_t) flags.doff;
        tcp->res1= (u_int16_t) flags.res1;
        tcp->res2= (u_int16_t) flags.res2;
        if (flags.urg) tcp->urg++;
        if (flags.ack) tcp->ack++;
        if (flags.psh) tcp->psh++;
        if (flags.rst) tcp->rst++;
        if (flags.fin) tcp->fin++;
        if (flags.syn) tcp->syn++;
        /*
	if (flags.ece) tcp->ece++;
        if (flags.cwr) tcp->cwr++;
	*/
        tcp->ack_seq = htonl(settings.ack_seq);
        tcp->seq = htonl(settings.seq);
        tcp->window=htons(settings.window);
        tcp->source=htons(sport);
        tcp->dest=htons(dport);

        if (!nodata)
                memcpy(msg, data, len);

        tcp->check = in_cksum((unsigned short *)tcp , (sizeof(struct tcphdr) + len) , csum);

        if (sendto(sock , packet , sizeof(struct iphdr) + sizeof(struct tcphdr) + len ,0,(struct sockaddr *)&sockfd, sizeof(sockfd)) < 0) {
                perror("sendto() ");
                exit(-1);
        }
	/*close(sock);*/
        return(0);
}

int
ls_sendicmp(struct ls_ip *ip4,char *srcHost,char *dstHost, u_int8_t type, u_int8_t code,unsigned char *data,int binlen)
{

        struct iphdr *ip;
        struct icmphdr *icmp;
        unsigned char *msg;
        unsigned char packet[1500];

        int len=strlen(data);
        int nodata=0;

        if (!strlen(data))
                nodata++;
        if (binlen)
                len=binlen;

        if (len > ICMP_MAXDATALEN ) {
                fprintf(stdout,"Data Field too big.\n");
                exit(-1);
        }
	sock=ls_socket(dstHost);

	memset(packet, 0, 1500);

        ip = (struct iphdr *)packet;

        icmp = (struct icmphdr *)(packet + sizeof(struct iphdr));

        msg = (unsigned char *)(packet + sizeof(struct iphdr) + sizeof(struct icmphdr));

        setiphdr(ip,ip4,sizeof(struct iphdr) + sizeof(struct icmphdr), IPPROTO_ICMP,srcHost, dstHost,0,0);

        icmp->type = type;
        icmp->code = code;

        if (!nodata)
                memcpy(msg, data, len);

        icmp->checksum = in_cksum((unsigned short *)icmp , (sizeof(struct icmphdr) + len) , 0);

        if (sendto(sock , packet ,(sizeof(struct icmphdr) + sizeof(struct iphdr) + len) ,0,(struct sockaddr *)&sockfd, sizeof(sockfd)) < 0) {
	        perror("sendto() ");
                exit(-1);
        }
	/*close(sock);*/
        return(0);
}

int
ls_sendudp(struct ls_ip *ip4,char *srcHost,char *dstHost,int sport, int dport,unsigned char *data,int binlen)
{

        struct pseudoIP psdIP;
        struct iphdr  *ip;
        struct udphdr *udp;

	unsigned char *msg;
        unsigned char packet[1500];
        unsigned short *psdHdr;
        unsigned int csum=0;
        int len=strlen(data);

        int nodata=0;

        if (!strlen(data))
                nodata++;

        if (binlen)
                len=binlen;

        sock=ls_socket(dstHost);
        if (len > UDP_MAXDATALEN ) {
                printf("Data Field too big.\n");
                exit(-1);
        }

        memset(packet, 0, 1500);

        ip = (struct iphdr *)packet;

        udp = (struct udphdr *)(packet + sizeof(struct iphdr));

        msg = (unsigned char *)(packet + sizeof(struct iphdr) + sizeof(struct udphdr));

        if ((inet_addr(dstHost) == INADDR_NONE) || (inet_addr(srcHost) == INADDR_NONE))
                exit(-1);

        setiphdr(ip,ip4,sizeof(struct iphdr) + sizeof(struct udphdr), IPPROTO_UDP,srcHost, dstHost,0,0);

        memset(&psdIP, 0, sizeof(struct pseudoIP));

        psdIP.source=ip->saddr;
        psdIP.dest=ip->daddr;
        psdIP.zero_byte=0;
        psdIP.protocol=IPPROTO_UDP;
        psdIP.len=htons(sizeof(struct udphdr) + len);

        psdHdr=(unsigned short * )&psdIP;
        csum = (psdHdr[0] + psdHdr[1] + psdHdr[2] + psdHdr[3] + psdHdr[4] + psdHdr[5]);

        udp->source=htons(sport);
        udp->dest=htons(dport);
        udp->len=htons( sizeof(struct udphdr) + len );

        if (!nodata)
                memcpy(msg, data, len);

        udp->check = in_cksum((unsigned short *)udp ,  (sizeof(struct udphdr) + len) , csum);

        if (sendto(sock , packet ,(sizeof(struct udphdr) + sizeof(struct iphdr) + len) ,0,(struct sockaddr *)&sockfd, sizeof(sockfd)) < 0) {
                perror("sendto() ");
                exit(-1);
        }
	/*close(sock);*/
        return(0);
}



void
setiphdr(struct iphdr *ip, struct ls_ip *ip4, int iplen, int protocol,char *srcHost, char *dstHost,int frag, int datalen)
{
	ip->version = 4;
        ip->ihl = 5;
        ip->tos = ip4->tos;
        ip->tot_len = htons(iplen);
        ip->id = htons(ip4->id);
        ip->frag_off = ip4->frag_off;
        ip->ttl = ip4->ttl;
        ip->protocol = protocol;
        ip->daddr = inet_addr(dstHost);
        ip->saddr = inet_addr(srcHost);
        ip->check = in_cksum((unsigned short *) &ip , 20,0);
}
/*******************************{ IPV6 }*******************************/
void
setiphdr6(struct ip6_hdr *ip6, int iplen, int protocol,char *srcHost, char *dstHost,int frag, int datalen,struct ls_v6opt option){

	inet_pton(AF_INET6,srcHost,(struct sockaddr_in6 *)&ip6->ip6_src) ;
	inet_pton(AF_INET6,dstHost,(struct sockaddr_in6 *)&ip6->ip6_dst) ;
       	ip6->ip6_vfc = htons(option.vfc);
	ip6->ip6_flow = htonl(option.flow_label);
	ip6->ip6_plen = htons(iplen);
	ip6->ip6_nxt = protocol;
	ip6->ip6_hlim = htons(option.hop_limit);
}
int
ls_socket6(char *dstHost, char *device){
	struct ifreq interface;
	struct sockaddr_in6 dst6;
	int sockfd;

	memset(&dst6, 0, sizeof(struct sockaddr_in6));
        inet_pton(AF_INET6,dstHost,(struct sockaddr_in6 *)&dst6.sin6_addr) ;
	dst6.sin6_family=AF_INET6;

	if ((sockfd = socket(AF_INET6, SOCK_RAW, 255)) < 0) {
                perror("socket() ");
                exit(-1);
                }
	memset(&interface, 0, sizeof(interface));
	strncpy(interface.ifr_name, device, strlen(device));

	if (setsockopt(sockfd, SOL_SOCKET , SO_BINDTODEVICE, &interface, sizeof(interface)) <0 ){
	       perror("socket option() ");
	       exit(-1);
	       }
	return(sockfd);
}

int
ls_sendudp6(char *srcHost,char *dstHost,int sport, int dport,unsigned char *data, int binlen,struct ls_v6opt option){

	struct ip6_hdr *ip6;
	struct udphdr *udp;
	struct pseudoIP6 phdr;
	struct sockaddr_in6 src6;

        int csum,c,len=strlen(data);
	unsigned char packet[1500];
	unsigned short *ptr;
	char *msg;

        int nodata=0;

        if (!strlen(data))
                nodata++;

        if (binlen)
                len=binlen;

        if (len > UDP6_MAXDATALEN ) {
                printf("Data Field too big.\n");
                exit(-1);
        }

	memset(packet, 0, 1500);
	ip6 = (struct ip6_hdr *)packet;
	udp = (struct udphdr *)(packet + sizeof(struct ip6_hdr));
	msg = (char *)(packet + sizeof(struct ip6_hdr) + sizeof(struct udphdr));

	if (!nodata)
                memcpy(msg, data, len);

	memset(&src6, 0, sizeof(struct sockaddr_in6));
        inet_pton(AF_INET6,srcHost,(struct sockaddr_in6 *)&src6.sin6_addr) ;
	src6.sin6_family=AF_INET6;
	src6.sin6_port=htons(0);
	/*
	fprintf(stdout,"sport: %d\n",sport);
	fprintf(stdout,"dport: %d\n",dport);
	*/
	sock=ls_socket6(dstHost,option.device);

	setiphdr6(ip6,sizeof(struct udphdr) + len, IPPROTO_UDP,srcHost, dstHost,0,0,option);

	memcpy(&phdr.src, &(ip6->ip6_src), sizeof(phdr.src));
	memcpy(&phdr.dst, &(ip6->ip6_dst), sizeof(phdr.dst));
	phdr.ulpl = htons(ip6->ip6_plen);
	phdr.zero_nxt = IPPROTO_UDP;
	ptr = (unsigned short *)&phdr;
	for(c=0; c < 20; c++)
		csum += ptr[c];

	udp->source=htons(sport);
        udp->dest=htons(dport);
        udp->len=htons( sizeof(struct udphdr) + len);
	udp->check = in_cksum((unsigned short *)udp ,  (sizeof(struct udphdr) + len) , csum);

	if (sendto(sock , packet ,(sizeof(struct udphdr) + sizeof(struct ip6_hdr) +len) ,0,(struct sockaddr *)&src6, sizeof(src6)) < 0) {
                perror("sendto() ");
                exit(-1);
        }
	/*close(sock);*/
	return(0);
}

int
ls_sendtcp6(char *srcHost,char *dstHost,int sport, int dport,unsigned char *data, int binlen, struct ls_tcpflags flags, struct ls_settings settings, struct ls_v6opt option){

	struct ip6_hdr *ip6;
	struct tcphdr *tcp;
	struct pseudoIP6 phdr;
	struct sockaddr_in6 src6;

        int csum,c,len=strlen(data);
	unsigned char packet[1500];
	unsigned short *ptr;
	char *msg;

        int nodata=0;

        if (!strlen(data))
                nodata++;

        if (binlen)
                len=binlen;

        if (len > UDP6_MAXDATALEN ) {
                printf("Data Field too big.\n");
                exit(-1);
        }

	memset(packet, 0, 1500);
	ip6 = (struct ip6_hdr *)packet;
	tcp = (struct tcphdr *)(packet + sizeof(struct ip6_hdr));
	msg = (char *)(packet + sizeof(struct ip6_hdr) + sizeof(struct tcphdr));

	if (!nodata)
                memcpy(msg, data, len);

	memset(&src6, 0, sizeof(struct sockaddr_in6));

        inet_pton(AF_INET6,srcHost,(struct sockaddr_in6 *)&src6.sin6_addr) ;
	src6.sin6_family=AF_INET6;
	src6.sin6_port=htons(0);

	sock=ls_socket6(dstHost,option.device);

	setiphdr6(ip6,sizeof(struct tcphdr)+len, IPPROTO_TCP,srcHost, dstHost,0,0,option);

	memcpy(&phdr.src, &(ip6->ip6_src), sizeof(phdr.src));
	memcpy(&phdr.dst, &(ip6->ip6_dst), sizeof(phdr.dst));
	phdr.ulpl = htons(ip6->ip6_plen);
	phdr.zero_nxt = IPPROTO_TCP;
	ptr = (unsigned short *)&phdr;
	for(c=0; c < 20; c++)
		csum += ptr[c];

        tcp->doff=(u_int16_t) flags.doff;
        tcp->res1= (u_int16_t) flags.res1;
        tcp->res2= (u_int16_t) flags.res2;
        if (flags.urg) tcp->urg++;
        if (flags.ack) tcp->ack++;
        if (flags.psh) tcp->psh++;
        if (flags.rst) tcp->rst++;
        if (flags.fin) tcp->fin++;
        if (flags.syn) tcp->syn++;
        tcp->ack_seq = htonl(settings.ack_seq);
        tcp->seq = htonl(settings.seq);
        tcp->window=htons(settings.window);
        tcp->source=htons(sport);
        tcp->dest=htons(dport);


        tcp->check = in_cksum((unsigned short *)tcp , (sizeof(struct tcphdr) + len) , csum);

	if (sendto(sock , packet ,(sizeof(struct ip6_hdr) + sizeof(struct tcphdr) + len) ,0,(struct sockaddr *)&src6,(socklen_t)sizeof(struct sockaddr_in6)) < 0) {
                perror("sendto() ");
                exit(-1);
        }
	/*close(sock);*/
	return(0);
}

int
ls_sendicmp6(char *srcHost,char *dstHost,struct ls_icmp6 icmpinfo,unsigned char *data, int binlen, struct ls_v6opt option){

	struct ip6_hdr *ip6;
	struct icmp6_hdr *icmp;
	struct sockaddr_in6 src6;

        int len=strlen(data);
	unsigned char packet[1500];
	char *msg;

        int nodata=0;

        if (!strlen(data))
                nodata++;

        if (binlen)
                len=binlen;

        if (len > ICMP6_MAXDATALEN ) {
                printf("Data Field too big.\n");
                exit(-1);
        }

	memset(packet, 0, 1500);
	ip6 = (struct ip6_hdr *)packet;
	icmp = (struct icmp6_hdr *)(packet + sizeof(struct ip6_hdr));
	msg = (char *)(packet + sizeof(struct ip6_hdr) + sizeof(struct icmp6_hdr));

	if (!nodata)
                memcpy(msg, data, len);

	memset(&src6, 0, sizeof(struct sockaddr_in6));

        inet_pton(AF_INET6,srcHost,(struct sockaddr_in6 *)&src6.sin6_addr) ;
	src6.sin6_family=AF_INET6;
	src6.sin6_port=htons(0);

	sock=ls_socket6(dstHost,option.device);

	setiphdr6(ip6,sizeof(struct icmp6_hdr)+len, IPPROTO_ICMPV6,srcHost, dstHost,0,0,option);

	icmp->icmp6_type=icmpinfo.type;
	icmp->icmp6_code=icmpinfo.code;
	icmp->icmp6_id=icmpinfo.id;
	icmp->icmp6_seq=icmpinfo.seq;
	icmp->icmp6_mtu=icmpinfo.mtu;
	icmp->icmp6_pptr=icmpinfo.pptr;

        icmp->icmp6_cksum = in_cksum((unsigned short *)icmp , (sizeof(struct icmp6_hdr) + len) , 0);

	if (sendto(sock , packet ,(sizeof(struct ip6_hdr) + sizeof(struct icmp6_hdr) + len) ,0,(struct sockaddr *)&src6,(socklen_t)sizeof(struct sockaddr_in6)) < 0) {
                perror("sendto() ");
                exit(-1);
        }
	/*close(sock);*/
	return(0);
}

unsigned short
in_cksum(unsigned short *addr,register unsigned int len,int csum)
{
        int nleft=len;
        int sum=csum;
        unsigned short *w = addr;
        unsigned short answer=0;

        while( nleft > 1 )
                {
                sum += *w++;
                nleft -= 2;
                }

        if (nleft == 1)
                {
                *(unsigned char *) (&answer) = *(unsigned char *)w;
                sum +=answer;
                }

        sum=(sum >> 16) + (sum & 0xffff );
        sum += (sum >> 16);
        answer = ~sum;
        return(answer);
        }
/*
 **************************
 *****[-Arp Section-]******
 **************************
 ** Coded by Lupsyn      **
 ** lupsyn@mojodo.it     **
 ** http://www.mojodo.it **
 **************************
 */
void die(char* str)
{
	fprintf(stderr,"\n[***] Attention exit for the followent reason : %s\n",str);
	exit(0);
}



int getdevindex(int sock, char *name)
{
        struct ifreq ifr;

        memset(&ifr, 0, sizeof(ifr));
        strncpy(ifr.ifr_name, name, sizeof(ifr.ifr_name));
        if(ioctl(sock, SIOCGIFINDEX, &ifr) == -1)
               die("I can't get device flags");
	return ifr.ifr_ifindex;

}

/*int strip(unsigned char *dst, char *str)
{
        if(sscanf(str,"%d.%d.%d.%d",&dst[0],&dst[1],&dst[2],&dst[3])==4)
               return 0;
        return -1;
}*/



void ls_sendarp(struct ls_arp pkt, char *dev)
{
	struct sockaddr_ll to;
	int sock;

	if( (sock = socket(PF_PACKET,SOCK_RAW,htons(ETH_P_ALL))) == -1)  die("Can't create socket");
	if( (to.sll_ifindex = getdevindex(sock,dev)) == -1)  die("Can't get interface");

	//Eth header
	pkt.frame_type = htons(ETH_P_ARP);
	//arp
	pkt.hw_type = htons(ARPHRD_ETHER);
	pkt.prot_type = htons(ETH_P_IP);
	pkt.hw_addr_size = ETH_ALEN;
	pkt.prot_addr_size = 4;
	pkt.op=htons(pkt.op);

	//data
	//get_hw_addr(pkt.Dst_a_hw,dst_a_hw);
	//get_hw_addr(pkt.Src_a_hw,dst_a_hw);
	//strip(pkt.Src_a_ip,src_a_ip);
	//strip(pkt.Dst_a_ip,dst_a_ip);

	//write and send
	if(sendto(sock,&pkt,sizeof(pkt),0,(struct sockaddr*)&to,sizeof(to))<0) die ("I can't send the packet");

}
